@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2014-2016 Matthias Klumpp <matthias@tenstral.net>
+ * Copyright (C) 2014-2017 Matthias Klumpp <matthias@tenstral.net>
  *
  * Licensed under the GNU Lesser General Public License Version 2.1
  *
@@ -42,7 +42,6 @@
 #include "as-utils.h"
 #include "as-utils-private.h"
 #include "as-spdx.h"
-#include "as-xmldata.h"
 #include "as-component.h"
 #include "as-component-private.h"
 
@@ -376,11 +375,11 @@ as_validator_check_description_tag (AsValidator *validator, xmlNode* node, AsCom
 								cpt,
 								"The '%s' tag should not be localized in collection metadata. Localize the whole 'description' tag instead.");
 			}
-			if ((first_paragraph) && (strlen (node_content) < 100)) {
+			if ((first_paragraph) && (strlen (node_content) < 80)) {
 				as_validator_add_issue (validator, iter,
 							AS_ISSUE_IMPORTANCE_INFO,
 							AS_ISSUE_KIND_VALUE_ISSUE,
-							"First 'description/p' paragraph might be too short (< 100 characters).",
+							"First 'description/p' paragraph might be too short (< 80 characters).",
 							node_content);
 			}
 			first_paragraph = FALSE;
@@ -590,7 +589,7 @@ as_validator_validate_update_contact (AsValidator *validator, xmlNode *uc_node)
  * as_validator_validate_component_node:
  **/
 static AsComponent*
-as_validator_validate_component_node (AsValidator *validator, AsXMLData *xdt, xmlNode *root)
+as_validator_validate_component_node (AsValidator *validator, AsContext *ctx, xmlNode *root)
 {
 	xmlNode *iter;
 	AsComponent *cpt;
@@ -601,11 +600,11 @@ as_validator_validate_component_node (AsValidator *validator, AsXMLData *xdt, xm
 	gboolean has_metadata_license = FALSE;
 
 	found_tags = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
-	mode = as_xmldata_get_format_style (xdt);
+	mode = as_context_get_style (ctx);
 
 	/* validate the resulting AsComponent for sanity */
 	cpt = as_component_new ();
-	as_xmldata_parse_component_node (xdt, root, cpt, NULL);
+	as_component_load_from_xml (cpt, ctx, root, NULL);
 	as_validator_set_current_cpt (validator, cpt);
 
 	/* check if component type is valid */
@@ -703,9 +702,9 @@ as_validator_validate_component_node (AsValidator *validator, AsXMLData *xdt, xm
 			as_validator_check_appear_once (validator, iter, found_tags, cpt);
 			if (g_str_has_suffix (node_content, ".")) {
 				as_validator_add_issue (validator, iter,
-							AS_ISSUE_IMPORTANCE_INFO,
+							AS_ISSUE_IMPORTANCE_PEDANTIC,
 							AS_ISSUE_KIND_VALUE_ISSUE,
-							"The component name should not end with a \".\" [%s]",
+							"The component name should (likely) not end with a \".\" [%s]",
 							node_content);
 			}
 
@@ -827,6 +826,7 @@ as_validator_validate_component_node (AsValidator *validator, AsXMLData *xdt, xm
 							AS_ISSUE_KIND_VALUE_WRONG,
 							"Unknown type '%s' for <translation/> tag.", prop);
 			}
+		} else if (g_strcmp0 (node_name, "launchable") == 0) {
 		} else if (g_strcmp0 (node_name, "extends") == 0) {
 		} else if (g_strcmp0 (node_name, "bundle") == 0) {
 			g_autofree gchar *prop = NULL;
@@ -849,6 +849,8 @@ as_validator_validate_component_node (AsValidator *validator, AsXMLData *xdt, xm
 			}
 		} else if (g_strcmp0 (node_name, "suggests") == 0) {
 			as_validator_check_children_quick (validator, iter, "id", cpt);
+		} else if (g_strcmp0 (node_name, "content_rating") == 0) {
+			as_validator_check_children_quick (validator, iter, "content_attribute", cpt);
 		} else if (g_strcmp0 (node_name, "custom") == 0) {
 			as_validator_check_appear_once (validator, iter, found_tags, cpt);
 			as_validator_check_children_quick (validator, iter, "value", cpt);
@@ -977,6 +979,16 @@ as_validator_validate_component_node (AsValidator *validator, AsXMLData *xdt, xm
 		}
 	}
 
+	/* validate service specific stuff */
+	if (as_component_get_kind (cpt) == AS_COMPONENT_KIND_SERVICE) {
+		AsLaunchable *launch = as_component_get_launchable (cpt, AS_LAUNCHABLE_KIND_SERVICE);
+		if (launch == NULL || as_launchable_get_entries (launch)->len == 0)
+			as_validator_add_issue (validator, NULL,
+					AS_ISSUE_IMPORTANCE_ERROR,
+					AS_ISSUE_KIND_TAG_MISSING,
+					"This 'service' component is missing a 'launchable' tag of type 'service'.");
+	}
+
 	/* validate suggestions */
 	if (as_component_get_suggested (cpt)->len > 0) {
 		guint j;
@@ -1008,7 +1020,7 @@ as_validator_validate_component_node (AsValidator *validator, AsXMLData *xdt, xm
 				as_validator_add_issue (validator, NULL,
 							AS_ISSUE_IMPORTANCE_WARNING,
 							AS_ISSUE_KIND_VALUE_WRONG,
-							"The category '%s' defined is not valid. Refer to the Freedesktop menu specificaton for a list of valid categories.", category_name);
+							"The category '%s' defined is not valid. Refer to the Freedesktop menu specification for a list of valid categories.", category_name);
 			}
 		}
 	}
@@ -1122,12 +1134,12 @@ as_validator_validate_file (AsValidator *validator, GFile *metadata_file)
  * as_validator_open_xml_document:
  */
 static xmlDoc*
-as_validator_open_xml_document (AsValidator *validator, AsXMLData *xdt, const gchar *xmldata)
+as_validator_open_xml_document (AsValidator *validator, const gchar *xmldata)
 {
 	xmlDoc *doc;
 	g_autoptr(GError) error = NULL;
 
-	doc = as_xmldata_parse_document (xdt, xmldata, &error);
+	doc = as_xml_parse_document (xmldata, &error);
 	if (doc == NULL) {
 		if (error != NULL) {
 			as_validator_add_issue (validator, NULL,
@@ -1155,41 +1167,36 @@ as_validator_validate_data (AsValidator *validator, const gchar *metadata)
 	gboolean ret;
 	xmlNode* root;
 	xmlDoc *doc;
-	g_autoptr(AsXMLData) xdt = NULL;
+	g_autoptr(AsContext) ctx = NULL;
 	AsComponent *cpt;
 
 	/* load the XML data */
-	xdt = as_xmldata_new ();
-	as_xmldata_initialize (xdt, AS_CURRENT_FORMAT_VERSION,
-			       "C",
-				NULL,
-				NULL,
-				NULL,
-				0);
+	ctx = as_context_new ();
+	as_context_set_locale (ctx, "C");
 
-	doc = as_validator_open_xml_document (validator, xdt, metadata);
+	doc = as_validator_open_xml_document (validator, metadata);
 	if (doc == NULL)
 		return FALSE;
 	root = xmlDocGetRootElement (doc);
 
 	ret = TRUE;
 	if (g_strcmp0 ((gchar*) root->name, "component") == 0) {
-		as_xmldata_set_format_style (xdt, AS_FORMAT_STYLE_METAINFO);
-		cpt = as_validator_validate_component_node (validator, xdt, root);
+		as_context_set_style (ctx, AS_FORMAT_STYLE_METAINFO);
+		cpt = as_validator_validate_component_node (validator, ctx, root);
 		if (cpt != NULL)
 			g_object_unref (cpt);
 	} else if (g_strcmp0 ((gchar*) root->name, "components") == 0) {
 		xmlNode *iter;
 		const gchar *node_name;
 
-		as_xmldata_set_format_style (xdt, AS_FORMAT_STYLE_COLLECTION);
+		as_context_set_style (ctx, AS_FORMAT_STYLE_COLLECTION);
 		for (iter = root->children; iter != NULL; iter = iter->next) {
 			/* discard spaces */
 			if (iter->type != XML_ELEMENT_NODE)
 				continue;
 			node_name = (const gchar*) iter->name;
 			if (g_strcmp0 (node_name, "component") == 0) {
-				cpt = as_validator_validate_component_node (validator, xdt, iter);
+				cpt = as_validator_validate_component_node (validator, ctx, iter);
 				if (cpt != NULL)
 					g_object_unref (cpt);
 			} else {
@@ -1288,53 +1295,58 @@ as_validator_analyze_component_metainfo_relation_cb (const gchar *fname, AsCompo
 
 	/* check if the referenced .desktop file exists */
 	if (as_component_get_kind (cpt) == AS_COMPONENT_KIND_DESKTOP_APP) {
-		if (g_hash_table_contains (data->desktop_fnames, as_component_get_desktop_id (cpt))) {
-			g_autofree gchar *desktop_fname_full = NULL;
-			g_autoptr(GKeyFile) dfile = NULL;
-			GError *tmp_error = NULL;
+		AsLaunchable *de_launchable = as_component_get_launchable (cpt, AS_LAUNCHABLE_KIND_DESKTOP_ID);
+		if ((de_launchable != NULL) && (as_launchable_get_entries (de_launchable)->len > 0)) {
+			const gchar *desktop_id = g_ptr_array_index (as_launchable_get_entries (de_launchable), 0);
 
-			desktop_fname_full = g_build_filename (data->apps_dir, as_component_get_desktop_id (cpt), NULL);
-			dfile = g_key_file_new ();
+			if (g_hash_table_contains (data->desktop_fnames, desktop_id)) {
+				g_autofree gchar *desktop_fname_full = NULL;
+				g_autoptr(GKeyFile) dfile = NULL;
+				GError *tmp_error = NULL;
 
-			g_key_file_load_from_file (dfile, desktop_fname_full, G_KEY_FILE_NONE, &tmp_error);
-			if (tmp_error != NULL) {
-				as_validator_add_issue (data->validator, NULL,
-						AS_ISSUE_IMPORTANCE_WARNING,
-						AS_ISSUE_KIND_READ_ERROR,
-						"Unable to read associated .desktop file: %s", tmp_error->message);
-				g_error_free (tmp_error);
-				tmp_error = NULL;
-			} else {
-				/* we successfully opened the .desktop file, now perform some checks */
+				desktop_fname_full = g_build_filename (data->apps_dir, desktop_id, NULL);
+				dfile = g_key_file_new ();
 
-				/* categories */
-				if (g_key_file_has_key (dfile, G_KEY_FILE_DESKTOP_GROUP,
-								G_KEY_FILE_DESKTOP_KEY_CATEGORIES, NULL)) {
-					g_autofree gchar *cats_str = NULL;
-					g_auto(GStrv) cats = NULL;
-					guint i;
+				g_key_file_load_from_file (dfile, desktop_fname_full, G_KEY_FILE_NONE, &tmp_error);
+				if (tmp_error != NULL) {
+					as_validator_add_issue (data->validator, NULL,
+							AS_ISSUE_IMPORTANCE_WARNING,
+							AS_ISSUE_KIND_READ_ERROR,
+							"Unable to read associated .desktop file: %s", tmp_error->message);
+					g_error_free (tmp_error);
+					tmp_error = NULL;
+				} else {
+					/* we successfully opened the .desktop file, now perform some checks */
 
-					cats_str = g_key_file_get_string (dfile, G_KEY_FILE_DESKTOP_GROUP,
-										 G_KEY_FILE_DESKTOP_KEY_CATEGORIES, NULL);
-					cats = g_strsplit (cats_str, ";", -1);
-					for (i = 0; cats[i] != NULL; i++) {
-						if (as_str_empty (cats[i]))
-							continue;
-						if (!as_utils_is_category_name (cats[i])) {
-							as_validator_add_issue (data->validator, NULL,
-										AS_ISSUE_IMPORTANCE_WARNING,
-										AS_ISSUE_KIND_VALUE_WRONG,
-										"The category '%s' defined in the .desktop file does not exist.", cats[i]);
+					/* categories */
+					if (g_key_file_has_key (dfile, G_KEY_FILE_DESKTOP_GROUP,
+									G_KEY_FILE_DESKTOP_KEY_CATEGORIES, NULL)) {
+						g_autofree gchar *cats_str = NULL;
+						g_auto(GStrv) cats = NULL;
+						guint i;
+
+						cats_str = g_key_file_get_string (dfile, G_KEY_FILE_DESKTOP_GROUP,
+											G_KEY_FILE_DESKTOP_KEY_CATEGORIES, NULL);
+						cats = g_strsplit (cats_str, ";", -1);
+						for (i = 0; cats[i] != NULL; i++) {
+							if (as_str_empty (cats[i]))
+								continue;
+							if (!as_utils_is_category_name (cats[i])) {
+								as_validator_add_issue (data->validator, NULL,
+											AS_ISSUE_IMPORTANCE_WARNING,
+											AS_ISSUE_KIND_VALUE_WRONG,
+											"The category '%s' defined in the .desktop file does not exist.", cats[i]);
+							}
 						}
 					}
-				}
 
+				}
+			} else {
+				as_validator_add_issue (data->validator, NULL,
+						AS_ISSUE_IMPORTANCE_ERROR,
+						AS_ISSUE_KIND_FILE_MISSING,
+						"Component metadata refers to a non-existing .desktop file.");
 			}
-		} else {
-			as_validator_add_issue (data->validator, NULL,
-					AS_ISSUE_IMPORTANCE_ERROR,
-					AS_ISSUE_KIND_FILE_MISSING,
-					"Component metadata refers to a non-existing .desktop file.");
 		}
 	}
 
@@ -1362,7 +1374,7 @@ as_validator_validate_tree (AsValidator *validator, const gchar *root_dir)
 	GHashTable *validated_cpts = NULL;
 	guint i;
 	gboolean ret = TRUE;
-	g_autoptr(AsXMLData) xdt = NULL;
+	g_autoptr(AsContext) ctx = NULL;
 	struct MInfoCheckData ht_helper;
 
 	/* cleanup */
@@ -1397,14 +1409,9 @@ as_validator_validate_tree (AsValidator *validator, const gchar *root_dir)
 						g_object_unref);
 
 	/* set up XML parser */
-	xdt = as_xmldata_new ();
-	as_xmldata_initialize (xdt, AS_CURRENT_FORMAT_VERSION,
-			       "C",
-				NULL,
-				NULL,
-				NULL,
-				0);
-	as_xmldata_set_format_style (xdt, AS_FORMAT_STYLE_METAINFO);
+	ctx = as_context_new ();
+	as_context_set_locale (ctx, "C");
+	as_context_set_style (ctx, AS_FORMAT_STYLE_METAINFO);
 
 	/* validate all metainfo files */
 	mfiles = as_utils_find_files_matching (metainfo_dir, "*.xml", FALSE, NULL);
@@ -1482,7 +1489,7 @@ as_validator_validate_tree (AsValidator *validator, const gchar *root_dir)
 		}
 
 		/* now read the XML */
-		doc = as_validator_open_xml_document (validator, xdt, asdata->str);
+		doc = as_validator_open_xml_document (validator, asdata->str);
 		if (doc == NULL) {
 			as_validator_clear_current_fname (validator);
 			continue;
@@ -1492,7 +1499,7 @@ as_validator_validate_tree (AsValidator *validator, const gchar *root_dir)
 		if (g_strcmp0 ((gchar*) root->name, "component") == 0) {
 			AsComponent *cpt;
 			cpt = as_validator_validate_component_node (validator,
-								    xdt,
+								    ctx,
 								    root);
 			if (cpt != NULL)
 				g_hash_table_insert (validated_cpts,
