@@ -22,7 +22,7 @@
 #include <glib/gprintf.h>
 
 #include "appstream.h"
-#include "as-xmldata.h"
+#include "as-xml.h"
 #include "as-component-private.h"
 #include "as-test-utils.h"
 
@@ -250,6 +250,58 @@ test_appstream_write_locale ()
 	g_object_unref (metad);
 }
 
+/**
+ * as_xml_test_read_data:
+ *
+ * Helper function for other tests.
+ */
+static AsComponent*
+as_xml_test_read_data (const gchar *data, AsFormatStyle mode)
+{
+	AsComponent *cpt = NULL;
+	GPtrArray *cpts;
+	g_autoptr(AsMetadata) metad = NULL;
+	g_autoptr(GError) error = NULL;
+
+	metad = as_metadata_new ();
+	as_metadata_parse (metad, data, AS_FORMAT_KIND_XML, &error);
+	g_assert_no_error (error);
+
+	cpts = as_metadata_get_components (metad);
+	g_assert_cmpint (cpts->len, >, 0);
+	if (mode == AS_FORMAT_STYLE_METAINFO)
+		g_assert_cmpint (cpts->len, ==, 1);
+	cpt = AS_COMPONENT (g_ptr_array_index (cpts, 0));
+
+	return g_object_ref (cpt);
+}
+
+/**
+ * as_xml_test_serialize:
+ *
+ * Helper function for other tests.
+ */
+static gchar*
+as_xml_test_serialize (AsComponent *cpt, AsFormatStyle mode)
+{
+	gchar *data;
+	g_autoptr(AsMetadata) metad = NULL;
+	g_autoptr(GError) error = NULL;
+
+	metad = as_metadata_new ();
+	as_metadata_add_component (metad, cpt);
+
+	if (mode == AS_FORMAT_STYLE_METAINFO) {
+		data = as_metadata_component_to_metainfo (metad, AS_FORMAT_KIND_XML, &error);
+		g_assert_no_error (error);
+	} else {
+		data = as_metadata_components_to_collection (metad, AS_FORMAT_KIND_XML, &error);
+		g_assert_no_error (error);
+	}
+
+	return data;
+}
+
 /**
  * test_appstream_write_description:
  *
@@ -331,7 +383,7 @@ test_appstream_write_description ()
 						"</component>\n";
 
 	const gchar *EXPECTED_XML_DISTRO = "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n"
-					   "<components version=\"0.10\">\n"
+					   "<components version=\"0.11\">\n"
 					   "  <component>\n"
 					   "    <id>org.example.Test</id>\n"
 					   "    <name>Test</name>\n"
@@ -430,57 +482,60 @@ test_appstream_write_description ()
 }
 
 /**
- * as_xml_test_read_data:
+ * test_appstream_read_description:
  *
- * Helper function for other tests.
+ * Test reading the description tag.
  */
-static AsComponent*
-as_xml_test_read_data (const gchar *data, AsFormatStyle mode)
+static void
+test_appstream_read_description (void)
 {
-	AsComponent *cpt;
-	GError *error = NULL;
-	g_autoptr(GPtrArray) cpts = NULL;
-	g_autoptr(AsXMLData) xdt = NULL;
+	g_autoptr(AsComponent) cpt = NULL;
+	const gchar *xmldata_desc_mi1 = "<component>\n"
+					"  <id>org.example.DescTestMI-1</id>\n"
+					"  <description>\n"
+					"    <p>Agenda is a simple, slick, speedy and no-nonsense task manager. Use it to keep track of the tasks that matter most.</p>\n"
+					"    <ul>\n"
+					"      <li>Blazingly fast and light</li>\n"
+					"      <li>Remembers your list until you clear completed tasks</li>\n"
+					"      <li>...</li>\n"
+					"    </ul>\n"
+					"    <p>I dare you to find an easier, faster, more beautiful task manager for elementary OS.</p>\n"
+					"  </description>\n"
+					"</component>\n";
 
-	xdt = as_xmldata_new ();
-	as_xmldata_set_check_valid (xdt, FALSE);
+	const gchar *xmldata_desc_mi2 = "<component>\n"
+					"  <id>org.example.DescTestMI-2</id>\n"
+					"  <description>\n"
+					"    <ul>\n"
+					"      <li>I start with bullet points</li>\n"
+					"      <li>Yes, this is allowed now</li>\n"
+					"      <li>...</li>\n"
+					"    </ul>\n"
+					"    <p>Paragraph</p>\n"
+					"  </description>\n"
+					"</component>\n";
 
-	if (mode == AS_FORMAT_STYLE_METAINFO) {
-		cpt = as_xmldata_parse_metainfo_data (xdt, data, &error);
-		g_assert_no_error (error);
-	} else {
-		cpts = as_xmldata_parse_collection_data (xdt, data, &error);
-		g_assert_no_error (error);
-		cpt = AS_COMPONENT (g_ptr_array_index (cpts, 0));
-	}
+	cpt = as_xml_test_read_data (xmldata_desc_mi1, AS_FORMAT_STYLE_METAINFO);
+	g_assert_cmpstr (as_component_get_id (cpt), ==, "org.example.DescTestMI-1");
 
-	return g_object_ref (cpt);
-}
+	g_assert_cmpstr (as_component_get_description (cpt), ==, "<p>Agenda is a simple, slick, speedy and no-nonsense task manager. Use it to keep track of the tasks that matter most.</p>\n"
+								 "<ul>\n"
+								 "  <li>Blazingly fast and light</li>\n"
+								 "  <li>Remembers your list until you clear completed tasks</li>\n"
+								 "  <li>...</li>\n"
+								 "</ul>\n"
+								 "<p>I dare you to find an easier, faster, more beautiful task manager for elementary OS.</p>\n");
 
-/**
- * as_xml_test_serialize:
- *
- * Helper function for other tests.
- */
-static gchar*
-as_xml_test_serialize (AsComponent *cpt, AsFormatStyle mode)
-{
-	gchar *data;
-	g_autoptr(AsXMLData) xdt = NULL;
+	g_object_unref (cpt);
+	cpt = as_xml_test_read_data (xmldata_desc_mi2, AS_FORMAT_STYLE_METAINFO);
+	g_assert_cmpstr (as_component_get_id (cpt), ==, "org.example.DescTestMI-2");
 
-	xdt = as_xmldata_new ();
-	as_xmldata_set_check_valid (xdt, FALSE);
-
-	if (mode == AS_FORMAT_STYLE_METAINFO) {
-		data = as_xmldata_serialize_to_metainfo (xdt, cpt);
-	} else {
-		g_autoptr(GPtrArray) cpts = NULL;
-		cpts = g_ptr_array_new ();
-		g_ptr_array_add (cpts, cpt);
-		data = as_xmldata_serialize_to_collection (xdt, cpts, TRUE);
-	}
-
-	return data;
+	g_assert_cmpstr (as_component_get_description (cpt), ==, "<ul>\n"
+								 "  <li>I start with bullet points</li>\n"
+								 "  <li>Yes, this is allowed now</li>\n"
+								 "  <li>...</li>\n"
+								 "</ul>\n"
+								 "<p>Paragraph</p>\n");
 }
 
 /**
@@ -671,7 +726,7 @@ test_xml_write_suggests (void)
 					"  </suggests>\n"
 					"</component>\n";
 	const gchar *expected_sug_xml_coll = "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n"
-					"<components version=\"0.10\">\n"
+					"<components version=\"0.11\">\n"
 					"  <component>\n"
 					"    <id>org.example.SuggestsTest</id>\n"
 					"    <suggests type=\"upstream\">\n"
@@ -755,6 +810,233 @@ test_xml_write_custom (void)
 	g_assert (as_test_compare_lines (res, xmldata_custom));
 }
 
+static const gchar *xmldata_content_rating = "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n"
+						"<component>\n"
+						"  <id>org.example.ContentRatingTest</id>\n"
+						"  <content_rating type=\"oars-1.0\">\n"
+						"    <content_attribute id=\"drugs-alcohol\">moderate</content_attribute>\n"
+						"    <content_attribute id=\"language-humor\">mild</content_attribute>\n"
+						"  </content_rating>\n"
+						"</component>\n";
+/**
+ * test_xml_read_content_rating:
+ *
+ * Test reading the content_rating tag.
+ */
+static void
+test_xml_read_content_rating (void)
+{
+	g_autoptr(AsComponent) cpt = NULL;
+	AsContentRating *rating;
+
+	cpt = as_xml_test_read_data (xmldata_content_rating, AS_FORMAT_STYLE_METAINFO);
+	g_assert_cmpstr (as_component_get_id (cpt), ==, "org.example.ContentRatingTest");
+
+	rating = as_component_get_content_rating (cpt, "oars-1.0");
+	g_assert_nonnull (rating);
+
+	g_assert_cmpint (as_content_rating_get_value (rating, "drugs-alcohol"), ==, AS_CONTENT_RATING_VALUE_MODERATE);
+	g_assert_cmpint (as_content_rating_get_value (rating, "language-humor"), ==, AS_CONTENT_RATING_VALUE_MILD);
+	g_assert_cmpint (as_content_rating_get_minimum_age (rating), ==, 13);
+}
+
+/**
+ * test_xml_write_content_rating:
+ *
+ * Test writing the content_rating tag.
+ */
+static void
+test_xml_write_content_rating (void)
+{
+	g_autoptr(AsComponent) cpt = NULL;
+	g_autoptr(AsContentRating) rating = NULL;
+	g_autofree gchar *res = NULL;
+
+	cpt = as_component_new ();
+	as_component_set_id (cpt, "org.example.ContentRatingTest");
+
+	rating = as_content_rating_new ();
+	as_content_rating_set_kind (rating, "oars-1.0");
+
+	as_content_rating_set_value (rating, "drugs-alcohol", AS_CONTENT_RATING_VALUE_MODERATE);
+	as_content_rating_set_value (rating, "language-humor", AS_CONTENT_RATING_VALUE_MILD);
+
+	as_component_add_content_rating (cpt, rating);
+
+	res = as_xml_test_serialize (cpt, AS_FORMAT_STYLE_METAINFO);
+	g_assert (as_test_compare_lines (res, xmldata_content_rating));
+}
+
+static const gchar *xmldata_launch = "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n"
+					"<component>\n"
+					"  <id>org.example.LaunchTest</id>\n"
+					"  <launchable type=\"desktop-id\">org.example.Test.desktop</launchable>\n"
+					"  <launchable type=\"desktop-id\">kde4-kool.desktop</launchable>\n"
+					"</component>\n";
+/**
+ * test_xml_read_launch:
+ *
+ * Test reading the launch tag.
+ */
+static void
+test_xml_read_launch (void)
+{
+	g_autoptr(AsComponent) cpt = NULL;
+	AsLaunchable *launch;
+
+	cpt = as_xml_test_read_data (xmldata_launch, AS_FORMAT_STYLE_METAINFO);
+	g_assert_cmpstr (as_component_get_id (cpt), ==, "org.example.LaunchTest");
+
+	launch = as_component_get_launchable (cpt, AS_LAUNCHABLE_KIND_DESKTOP_ID);
+	g_assert_nonnull (launch);
+
+	g_assert_cmpint (as_launchable_get_entries (launch)->len, ==, 2);
+	g_assert_cmpstr (g_ptr_array_index (as_launchable_get_entries (launch), 0), ==, "org.example.Test.desktop");
+	g_assert_cmpstr (g_ptr_array_index (as_launchable_get_entries (launch), 1), ==, "kde4-kool.desktop");
+}
+
+/**
+ * test_xml_write_launch:
+ *
+ * Test writing the launch tag.
+ */
+static void
+test_xml_write_launch (void)
+{
+	g_autoptr(AsComponent) cpt = NULL;
+	g_autoptr(AsLaunchable) launch = NULL;
+	g_autofree gchar *res = NULL;
+
+	cpt = as_component_new ();
+	as_component_set_id (cpt, "org.example.LaunchTest");
+
+	launch = as_launchable_new ();
+	as_launchable_set_kind (launch, AS_LAUNCHABLE_KIND_DESKTOP_ID);
+
+	as_launchable_add_entry (launch, "org.example.Test.desktop");
+	as_launchable_add_entry (launch, "kde4-kool.desktop");
+
+	as_component_add_launchable (cpt, launch);
+
+	res = as_xml_test_serialize (cpt, AS_FORMAT_STYLE_METAINFO);
+	g_assert (as_test_compare_lines (res, xmldata_launch));
+}
+
+/**
+ * test_appstream_write_metainfo_to_collection:
+ */
+static void
+test_appstream_write_metainfo_to_collection (void)
+{
+	gchar *tmp;
+	g_autoptr(AsMetadata) metad = NULL;
+	g_autoptr(GError) error = NULL;
+
+	const gchar *METAINFO_XML = "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n"
+					"<component>\n"
+					"  <id>org.example.Test</id>\n"
+					"  <name>Test</name>\n"
+					"  <name xml:lang=\"eo\">Testo</name>\n"
+					"  <name xml:lang=\"x-test\">I am a cruft entry</name>\n"
+					"  <name xml:lang=\"de\">Test</name>\n"
+					"  <summary>Just a unittest.</summary>\n"
+					"  <summary xml:lang=\"de\">Nur ein Unittest.</summary>\n"
+					"  <description>\n"
+					"    <p>First paragraph</p>\n"
+					"    <ol>\n"
+					"      <li>One</li>\n"
+					"      <li>Two</li>\n"
+					"      <li>Three is &gt; 2 &amp; 1</li>\n"
+					"    </ol>\n"
+					"    <p>Paragraph2</p>\n"
+					"    <ul>\n"
+					"      <li>First</li>\n"
+					"      <li>Second</li>\n"
+					"    </ul>\n"
+					"    <p>Paragraph3 &amp; the last one</p>\n"
+					"    <p xml:lang=\"de\">Erster Absatz</p>\n"
+					"    <ol>\n"
+					"      <li>One</li>\n"
+					"      <li xml:lang=\"de\">Eins</li>\n"
+					"      <li xml:lang=\"de\">Zwei</li>\n"
+					"      <li xml:lang=\"de\">Drei</li>\n"
+					"    </ol>\n"
+					"    <ul>\n"
+					"      <li xml:lang=\"de\">Erster</li>\n"
+					"      <li xml:lang=\"de\">Zweiter</li>\n"
+					"    </ul>\n"
+					"    <p xml:lang=\"de\">Absatz2</p>\n"
+					"  </description>\n"
+					"  <icon type=\"cached\" width=\"20\" height=\"20\">test_writetest.png</icon>\n"
+					"  <icon type=\"cached\" width=\"40\" height=\"40\">test_writetest.png</icon>\n"
+					"  <icon type=\"stock\">xml-writetest</icon>\n"
+					"  <releases>\n"
+					"    <release version=\"1.0\" date=\"2016-04-11T22:00:00+00:00\"/>\n"
+					"  </releases>\n"
+					"</component>\n";
+
+	const gchar *EXPECTED_XML_COLL = "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n"
+					   "<components version=\"0.11\">\n"
+					   "  <component>\n"
+					   "    <id>org.example.Test</id>\n"
+					   "    <name xml:lang=\"de\">Test</name>\n"
+					   "    <name>Test</name>\n"
+					   "    <name xml:lang=\"eo\">Testo</name>\n"
+					   "    <summary>Just a unittest.</summary>\n"
+					   "    <summary xml:lang=\"de\">Nur ein Unittest.</summary>\n"
+					   "    <description>\n"
+					   "      <p>First paragraph</p>\n"
+					   "      <ol>\n"
+					   "        <li>One</li>\n"
+					   "        <li>Two</li>\n"
+					   "        <li>Three is &gt; 2 &amp; 1</li>\n"
+					   "      </ol>\n"
+					   "      <p>Paragraph2</p>\n"
+					   "      <ul>\n"
+					   "        <li>First</li>\n"
+					   "        <li>Second</li>\n"
+					   "      </ul>\n"
+					   "      <p>Paragraph3 &amp; the last one</p>\n"
+					   "      <ol>\n"
+					   "        <li>One</li>\n"
+					   "      </ol>\n"
+					   "      <ul/>\n"
+					   "    </description>\n"
+					   "    <description xml:lang=\"de\">\n"
+					   "      <p>Erster Absatz</p>\n"
+					   "      <ol>\n"
+					   "        <li>Eins</li>\n"
+					   "        <li>Zwei</li>\n"
+					   "        <li>Drei</li>\n"
+					   "      </ol>\n"
+					   "      <ul>\n"
+					   "        <li>Erster</li>\n"
+					   "        <li>Zweiter</li>\n"
+					   "      </ul>\n"
+					   "      <p>Absatz2</p>\n"
+					   "    </description>\n"
+					   "    <icon type=\"cached\" width=\"20\" height=\"20\">test_writetest.png</icon>\n"
+					   "    <icon type=\"cached\" width=\"40\" height=\"40\">test_writetest.png</icon>\n"
+					   "    <icon type=\"stock\">xml-writetest</icon>\n"
+					   "    <releases>\n"
+					   "      <release version=\"1.0\" timestamp=\"1460332800\"/>\n"
+					   "    </releases>\n"
+					   "  </component>\n"
+					   "</components>\n";
+
+	metad = as_metadata_new ();
+	as_metadata_set_locale (metad, "ALL");
+
+	as_metadata_parse (metad, METAINFO_XML, AS_FORMAT_KIND_XML, &error);
+	g_assert_no_error (error);
+
+	as_metadata_set_format_style (metad, AS_FORMAT_STYLE_COLLECTION);
+
+	tmp = as_metadata_components_to_collection (metad, AS_FORMAT_KIND_XML, NULL);
+	g_assert (as_test_compare_lines (tmp, EXPECTED_XML_COLL));
+	g_free (tmp);
+}
+
 /**
  * main:
  */
@@ -783,6 +1065,8 @@ main (int argc, char **argv)
 	g_test_add_func ("/XML/LegacyData", test_appstream_parser_legacy);
 	g_test_add_func ("/XML/Read/ParserLocale", test_appstream_parser_locale);
 	g_test_add_func ("/XML/Write/WriterLocale", test_appstream_write_locale);
+
+	g_test_add_func ("/XML/Read/Description", test_appstream_read_description);
 	g_test_add_func ("/XML/Write/Description", test_appstream_write_description);
 
 	g_test_add_func ("/XML/Read/Url", test_xml_read_url);
@@ -797,6 +1081,14 @@ main (int argc, char **argv)
 	g_test_add_func ("/XML/Read/Custom", test_xml_read_custom);
 	g_test_add_func ("/XML/Write/Custom", test_xml_write_custom);
 
+	g_test_add_func ("/XML/Read/ContentRating", test_xml_read_content_rating);
+	g_test_add_func ("/XML/Write/ContentRating", test_xml_write_content_rating);
+
+	g_test_add_func ("/XML/Read/Launchable", test_xml_read_launch);
+	g_test_add_func ("/XML/Write/Launchable", test_xml_write_launch);
+
+	g_test_add_func ("/XML/Write/MetainfoToCollection", test_appstream_write_metainfo_to_collection);
+
 	ret = g_test_run ();
 	g_free (datadir);
 	return ret;
